--!parent
--!encoding=utf-8

--****************************************************************************
-- @file			LSW CORE LIB: JSON
-- @author   	Valentin Schmidt
-- @version  	0.1
--****************************************************************************

property jsHelper

----------------------------------------
-- @constructor
----------------------------------------
on new (me)
  jsHelper = me.$.include(me.libdir&"json.js") -- JS helper functions
  return me
end

----------------------------------------
-- JSON_DECODE
-- @param {string|byteArray} data
-- @return {any}
----------------------------------------
on decode (me, data)
	if ilk(data)=#byteArray then
		data.position = 1
		data = data.readRawString(data.length)
	end if
	return jsHelper.json_decode(data)
end

----------------------------------------
-- JSON_ENCODE
-- @param {any} c
-- @return {any}
----------------------------------------
on encode (me, o)
  case ilk(o) of  
    #void:
      return "null"
    #integer, #float:
      return string(o)
    #string:
      return QUOTE & jsHelper.json_escape_string(o) & QUOTE
    #list:
      res = []
      repeat with v in o
        res.add(me.encode(v))
      end repeat
      return "[" & me._implode(",", res) & "]"
    #propList:
      res = []
      cnt = count(o)
      repeat with i = 1 to cnt
        p = o.getPropAt(i)
        v = o[i]
        res.add( me.encode(p)&":"&me.encode(v) )
      end repeat
      return "{" & me._implode(",", res) & "}"
    #symbol:
      --return QUOTE&"_sym_"&string(o)&QUOTE
      return QUOTE & jsHelper.json_escape_string(string(o)) & QUOTE
      
    otherwise:
      -- put "ERROR: unsupported data type!"
      return QUOTE&"<UNSUPPORTED DATA TYPE: "&ilk(o)&">"&QUOTE
      
  end case
end  

----------------------------------------
-- 
----------------------------------------
on _implode (me, delim, l)
  str=""
  repeat with i=1 to l.count
    put l[i]&delim after str
  end repeat
  delete char (str.length-delim.length+1) to str.length of str
  return str
end
